/*
 *  ******************************************************************************
 *  *
 *  *
 *  * This program and the accompanying materials are made available under the
 *  * terms of the Apache License, Version 2.0 which is available at
 *  * https://www.apache.org/licenses/LICENSE-2.0.
 *  *
 *  *  See the NOTICE file distributed with this work for additional
 *  *  information regarding copyright ownership.
 *  * Unless required by applicable law or agreed to in writing, software
 *  * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 *  * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 *  * License for the specific language governing permissions and limitations
 *  * under the License.
 *  *
 *  * SPDX-License-Identifier: Apache-2.0
 *  *****************************************************************************
 */

package org.datavec.spark.transform.sparkfunction.sequence;

import org.apache.spark.api.java.function.Function2;
import org.datavec.api.Writable;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

public class DataFrameToSequenceMergeCombiner
                implements Function2<List<List<Writable>>, List<List<Writable>>, List<List<Writable>>> {

    @Override
    public List<List<Writable>> call(List<List<Writable>> v1, List<List<Writable>> v2) throws Exception {
        List<List<Writable>> out = new ArrayList<>(v1.size() + v2.size());
        out.addAll(v1);
        out.addAll(v2);
        Collections.sort(out, new Comparator<List<Writable>>() {
            @Override
            public int compare(List<Writable> o1, List<Writable> o2) {
                return Integer.compare(o1.get(1).toInt(), o2.get(1).toInt());
            }
        });
        return out;
    }
}
