/*
 *
 *    ******************************************************************************
 *    *
 *    * This program and the accompanying materials are made available under the
 *    * terms of the Apache License, Version 2.0 which is available at
 *    * https://www.apache.org/licenses/LICENSE-2.0.
 *    *
 *    *  See the NOTICE file distributed with this work for additional
 *    *  information regarding copyright ownership.
 *    * Unless required by applicable law or agreed to in writing, software
 *    * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 *    * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 *    * License for the specific language governing permissions and limitations
 *    * under the License.
 *    *
 *    * SPDX-License-Identifier: Apache-2.0
 *    *****************************************************************************
 *
 */

package net.brutex.ai.dnn.api;

/**
 * This is an "executable" ILayer, that is based on a {@link ILayerConfiguration}
 */
public interface ILayer {

  /**
   * Get the underlying configuration for this ILayer
   * @return configuration
   */
  ILayerConfiguration getLayerConfiguration();

  /**
   * Set the underlying layer configuration
   * @param conf The new configuration
   */
  void setLayerConfiguration(ILayerConfiguration conf);

  /**
   * An implementation should provide a method to validate the network
   * @return true if no errors found; false otherwise
   */
  boolean isValid();
}
