/*
 *
 *    ******************************************************************************
 *    *
 *    * This program and the accompanying materials are made available under the
 *    * terms of the Apache License, Version 2.0 which is available at
 *    * https://www.apache.org/licenses/LICENSE-2.0.
 *    *
 *    *  See the NOTICE file distributed with this work for additional
 *    *  information regarding copyright ownership.
 *    * Unless required by applicable law or agreed to in writing, software
 *    * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 *    * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 *    * License for the specific language governing permissions and limitations
 *    * under the License.
 *    *
 *    * SPDX-License-Identifier: Apache-2.0
 *    *****************************************************************************
 *
 */

package net.brutex.ai.dnn.api;

/**
 * Unit A unit often refers to the activation function in a layer by which the inputs are
 * transformed via a nonlinear activation function (for example by the logistic sigmoid function).
 * Usually, a unit has several incoming connections and several outgoing connections. However, units
 * can also be more complex, like long short-term memory (LSTM) units, which have multiple
 * activation functions with a distinct layout of connections to the nonlinear activation functions,
 * or maxout units, which compute the final output over an array of nonlinearly transformed input
 * values. Pooling, convolution, and other input transforming functions are usually not referred to
 * as units.
 * <p>
 * Artificial Neuron The term artificial neuron—or most often just neuron—is an equivalent term to
 * unit, but implies a close connection to neurobiology and the human brain while deep learning has
 * very little to do with the brain (for example, it is now thought that biological neurons are more
 * similar to entire multilayer perceptrons rather than a single unit in a neural network). The term
 * neuron was encouraged after the last AI winter to differentiate the more successful neural
 * network from the failing and abandoned perceptron. However, since the wild successes of deep
 * learning after 2012, the media often picked up on the term “neuron” and sought to explain deep
 * learning as mimicry of the human brain, which is very misleading and potentially dangerous for
 * the perception of the field of deep learning. Now the term neuron is discouraged and the more
 * descriptive term unit should be used instead.
 */
public interface IUnit {

}
