/*
 *
 *    ******************************************************************************
 *    *
 *    * This program and the accompanying materials are made available under the
 *    * terms of the Apache License, Version 2.0 which is available at
 *    * https://www.apache.org/licenses/LICENSE-2.0.
 *    *
 *    *  See the NOTICE file distributed with this work for additional
 *    *  information regarding copyright ownership.
 *    * Unless required by applicable law or agreed to in writing, software
 *    * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 *    * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 *    * License for the specific language governing permissions and limitations
 *    * under the License.
 *    *
 *    * SPDX-License-Identifier: Apache-2.0
 *    *****************************************************************************
 *
 */

package org.deeplearning4j.optimize.listeners;

import java.io.IOException;
import lombok.extern.slf4j.Slf4j;
import okhttp3.OkHttpClient;
import okhttp3.Request;
import okhttp3.Response;
import net.brutex.ai.dnn.api.IModel;
import org.deeplearning4j.optimize.api.BaseTrainingListener;

@Slf4j
public class ScoreToChartListener extends BaseTrainingListener {

  final String url = "http://bru5:8080/cavis-rest-1.0-SNAPSHOT.war/hello/hello-world?";
  final String seriesName;

  public ScoreToChartListener(String seriesName) {
    this.seriesName = seriesName;
  }

  @Override
  public void iterationDone(IModel model, int iteration, int epoch) {
    double score = model.getScore();
    String nurl = url+"s="+score+"&n="+seriesName;
    OkHttpClient client = new OkHttpClient();

    Request request = new Request.Builder()
        .url(nurl)
        .build();

    try {
      Response response = client.newCall(request).execute();
      log.debug(String.format("Did send score to chart at '%s'.", nurl));
      response.body().close();
    } catch (IOException e) {
      log.warn(String.format("Could not send score to chart at '%s' because %s", nurl, e.getMessage()));
    }
    //response.body().string();
    }

}
